@@ -9,6 +9,8 @@ import sys
 import textwrap
 import time
 
+import sys
+import subprocess
 from cloudmesh.burn.image import Image
 from cloudmesh.burn.sdcard import SDCard
 from cloudmesh.burn.usb import USB
@@ -17,7 +19,6 @@ from cloudmesh.burn.util import os_is_mac
 from cloudmesh.burn.util import os_is_pi
 from cloudmesh.burn.util import os_is_windows
 from cloudmesh.common.JobScript import JobScript
-from cloudmesh.common.Printer import Printer
 from cloudmesh.common.Shell import Shell
 from cloudmesh.common.StopWatch import StopWatch
 from cloudmesh.common.Tabulate import Printer
@@ -26,60 +27,10 @@ from cloudmesh.common.util import banner
 from cloudmesh.common.util import path_expand
 from cloudmesh.common.util import writefile
 from cloudmesh.common.util import yn_choice
-
-
-# TODO: make sure everything is compatible with --dryrun
-
-def sudo_writefile(filename, content, append=False):
-    """
-    Write a file into the file with the given filename using sudo
-    The file is first created in ~/.cloudmesh/tmp/tmp.txt
-
-    TODO: append is not implemented
-
-    :param filename: The filename
-    :type filename: str
-    :param content: The content of the file
-    :type content: str
-    :param append: if True the content will be appended to the existing file
-    :type append: bool
-    :return: The new content of the file
-    :rtype: str
-    """
-    os.system('mkdir -p ~/.cloudmesh/tmp')
-    tmp = "~/.cloudmesh/tmp/tmp.txt"
-
-    if append:
-        content = sudo_readfile(filename, split=False) + content
-
-    writefile(tmp, content)
-
-    result = subprocess.getstatusoutput(f"sudo cp {tmp} {filename}")
-
-    # If exit code is not 0
-    if result[0] != 0:
-        Console.warning(f"{filename} was not created correctly -> {result[1]}")
-
-    return result[1]
-
-
-def sudo_readfile(filename, split=True):
-    """
-    Reads the file specified via sudo by filename and returns its content
-
-    :param filename: The filename
-    :type filename: str
-    :param split: if True splits the lines and returns a list
-    :type split: bool
-    :return: either str or list of the file content
-    :rtype: str or list
-    """
-    result = subprocess.getoutput(f"sudo cat {filename}")
-
-    if split:
-        result = result.split('\n')
-
-    return result
+from cloudmesh.common.util import sudo_readfile
+from cloudmesh.common.util import sudo_writefile
+from cloudmesh.common.systeminfo import get_platform
+from cloudmesh.common.util import readfile
 
 
 # def dmesg():
@@ -99,6 +50,15 @@ def gen_strong_pass():
         string.punctuation
     return ''.join(random.choice(password_characters) for i in range(length))
 
+def windows_not_supported(f):
+    def wrapper(*args,**kwargs):
+        host = get_platform()
+        if host == "windows":
+            Console.error("Not supported on windows")
+            return ""
+        else:
+            return f(*args, **kwargs)
+    return wrapper
 
 # noinspection PyPep8
 class Burner(object):
@@ -117,6 +77,122 @@ class Burner(object):
         self.hostname = None
         self.keypath = None
 
+    @windows_not_supported
+    def check(self, device="/dev/sdX"):
+        """
+        This method checks what configurations are placed on the PI se card
+
+        @param device:
+        @type device:
+        @return:
+        @rtype:
+        """
+
+        data = {
+            "ssh": None,
+            "hostname": None,
+            "ip": None,
+            "password": None,
+            "shadow": None,
+            "wifi": None,
+            "psk": None,
+            "ssid": None,
+            "wifipassword": None
+        }
+
+        card = SDCard()
+
+        # ssh
+
+        try:
+            self.mount(device)
+            data["ssh"] = os.path.exists(card.boot_volume)
+        except Exception as e:
+            data["ssh"] = str(e)
+        # hostname
+
+        content = readfile(f"{card.root_volume}/etc/hostname").strip()
+        data['hostname'] = content
+
+        # ip
+
+        data["ip"] = "not yet implemented"
+
+        # passwod
+
+        data["password"] = "not yet implemented"
+
+        data["shadow"] = os.path.exists(f"{card.boot_volume}/etc/shadow")
+
+        # wifi
+
+        try:
+            location = f"{card.boot_volume}/wpa_supplicant.conf"
+            data["wifi"] = os.path.exists(location)
+
+            if data["wifi"]:
+                lines = readfile(location).splitlines()
+
+                for line in lines:
+                    for tag in ["ssid", "wifipassword", "psk"]:
+                        if f"{tag} =" in line:
+                            data[tag] = line.split("{tag} =")[1]
+
+            else:
+                data["wifi"] = False
+                data["ssid"] = None
+                data["wifipassword"] = None
+
+        except Exception as e:
+
+            data["wifi"] = False
+            data["ssid"] = None
+            data["wifipasswd"] = None
+
+        banner("Card Check")
+        print(Printer.attribute(
+            data,
+            order=[
+                "ssh",
+                "hostname",
+                "ip",
+                "password",
+                "shadow",
+                "wifi",
+                "psk",
+                "ssid",
+                "wifipassword"
+            ]
+        ))
+
+    @windows_not_supported
+    def firmware(self, action="check"):
+        """
+        Checks or update the firmware
+        :param action: the cations to be performed. It is "check" or "update"
+        :type action: str
+        """
+        if not os_is_pi():
+            Console.error("This command can only be run on a PI")
+        else:
+            if action == "check":
+                Console.error("To be implemented")
+
+                command = f"sudo rpi-eeprom-update"
+                print(command)
+                os.system(command)
+
+                os.system ("vcgencmd bootloader_version")
+
+                print("For more information see:")
+                print()
+                print("* https://www.raspberrypi.org/documentation/hardware/raspberrypi/booteeprom.md")
+                print()
+            elif action == "update":
+                os.system("sudo rpi-eeprom-update -a")
+                os.system("sudo reboot")
+
+    @windows_not_supported
     def shrink(self, image=None):
         if image is None:
             Console.error("Image must have a value")
@@ -125,9 +201,11 @@ class Burner(object):
         print(command)
         os.system(command)
 
-    def shrink_install(self):
+    @windows_not_supported
+    def install(self):
         """
         Installs /usr/local/bin/pishrink.sh
+        Installes parted
         @return:
         @rtype:
         """
@@ -139,6 +217,7 @@ class Burner(object):
                 wget https://raw.githubusercontent.com/Drewsif/PiShrink/master/pishrink.sh
                 chmod +x pishrink.sh
                 sudo mv pishrink.sh /usr/local/bin
+                sudo apt install parted -y > $HOME/tmp.log
                 """
 
             result = JobScript.execute(script)
@@ -147,6 +226,7 @@ class Burner(object):
         else:
             raise NotImplementedError
 
+    @windows_not_supported
     def backup(self, device=None, to_file=None, blocksize="4M"):
         if device is None:
             Console.error("Device must have a value")
@@ -160,41 +240,14 @@ class Burner(object):
             print(command)
             os.system(command)
 
-    def copy(self, device=None, from_file=None):
+    @windows_not_supported
+    def copy(self, device=None, from_file="latest"):
         if device is None:
             Console.error("Device must have a value")
-        if from_file is None:
-            Console.error("From file must have a value")
-        burner.burn_sdcard(from_file, device)
+        self.burn_sdcard(from_file, device)
 
-    def detect(self):
-        """
-        Detects if a USB card writer can be found. and just prints the result
-        """
-        # Clear dmesg table so that info doesn't get confused with previous detects
-        self.system('sudo dmesg -c')
-        banner("Detecting USB Card Writers(s)")
-
-        print("Make sure the USB Writers(s) is removed ...")
-        if not yn_choice("Is the writer(s) removed?"):
-            sys.exit()
-        usb_out = set(Shell.execute("lsusb").splitlines())
-        print("Now plug in the Writer(s) ...")
-        if not yn_choice("Is the writer(s) plugged in?"):
-            sys.exit()
-        usb_in = set(Shell.execute("lsusb").splitlines())
-
-        writer = usb_in - usb_out
-        if len(writer) == 0:
-            print(
-                "ERROR: we did not detect the device, make sure it is plugged.")
-            sys.exit()
-        else:
-            banner("Detected Card Writers")
-
-            print("\n".join(writer))
-            print()
 
+    @windows_not_supported
     def info(self, print_stdout=True):
         """
         Finds out information about USB devices
@@ -315,7 +368,7 @@ class Burner(object):
         if print_stdout:
 
             if os_is_linux():
-                card = SDCard(os="raspberry", host="ubuntu")
+                card = SDCard(card_os="raspberry")
                 m = card.ls()
 
                 banner("Mount points")
@@ -356,6 +409,7 @@ class Burner(object):
         #     print(command)
         # else:
         #     os.system(command)
+
         res = subprocess.getstatusoutput(command)
         # If exit code is not 0, warn user
         if res[0] != 0 and res[0] != 32:
@@ -364,7 +418,8 @@ class Burner(object):
 
         return res[1]
 
-    def burn_sdcard(self, image, device, blocksize="4M"):
+    @windows_not_supported
+    def burn_sdcard(self, tag="latest-lite", device=None, blocksize="4M"):
         """
         Burns the SD Card with an image
 
@@ -375,9 +430,25 @@ class Burner(object):
         :param blocksize: the blocksize used when writing, default 4M
         :type blocksize: str
         """
-        if os_is_pi():
 
-            image_path = Image(image).fullpath
+        Console.info("Burning...")
+        image = Image().find(tag=tag)
+
+        if image is None:
+            Console.error("No matching image found.")
+            return ""
+        elif len(image) > 1:
+            Console.error("Too manay images found")
+            print(Printer.write(image,
+                                order=["tag", "version"],
+                                header=["Tag", "Version"]))
+            return ""
+
+        image = image[0]
+
+        image_path = Image().directory + "/" + Image.get_name(image["url"]) + ".img"
+
+        if os_is_pi():
 
             command = f"sudo dd bs={blocksize} if={image_path} of={device}"
 
@@ -387,7 +458,6 @@ class Burner(object):
                 Console.error("The image could not be found")
                 sys.exit(1)
         elif os_is_linux():
-            image_path = Image(image).fullpath
 
             print(image_path)
             print(device)
@@ -399,15 +469,16 @@ class Burner(object):
             # find device
 
             command = f"dd if={image_path} |" \
-                      " pv -w 80 |" \
-                      " sudo dd of={device} bs={blocksize} conv=fsync status=progress"
+                      f" pv -w 80 |" \
+                      f" sudo dd of={device} bs={blocksize} conv=fsync status=progress"
             print(command)
             os.system(command)
 
         else:
             raise NotImplementedError("Only implemented to be run on a PI")
 
-    def set_hostname(self, hostname, mountpoint):
+    @windows_not_supported
+    def set_hostname(self, hostname):
         """
         Sets the hostname on the sd card
 
@@ -418,6 +489,14 @@ class Burner(object):
         :type mountpoint: str
         """
         self.hostname = hostname
+        if os_is_pi():
+            card = SDCard()
+        elif os_is_linux():
+            card = SDCard(card_os="linux")
+        else:
+            raise NotImplementedError
+
+        mountpoint = card.root_volume
         # write the new hostname to /etc/hostname
         if not self.dryrun:
             self.system(
@@ -435,7 +514,7 @@ class Burner(object):
             f = sudo_readfile(f'{mountpoint}/etc/hosts')
             # lines = [l for l in f.readlines()][:-1]  # ignore the last line
             lines = f[:-1]
-            newlastline = '127.0.1.1 ' + hostname + '\n'
+            newlastline = '\n127.0.1.1 ' + hostname + '\n'
 
         if not self.dryrun:
             new_hostsfile_contents = ''.join(lines) + newlastline
@@ -445,7 +524,37 @@ class Burner(object):
             print("Write to /etc/hosts")
             print('127.0.1.1 ' + hostname + '\n')
 
-    def set_static_ip(self, ip, mountpoint, iface="eth0", mask="16"):
+        # Adds the ip and hostname to /etc/hosts if it isn't already there.
+    def add_to_hosts(self, ip):
+        # with open('/etc/hosts', 'r') as host_file:
+        #     hosts = host_file.readlines()
+        hosts = sudo_readfile('/etc/hosts')
+
+        replaced = False
+        for i in range(len(hosts)):
+            ip_host = hosts[i].split()
+
+            if len(ip_host) > 1:
+                if ip_host[0] == ip:
+                    ip_host[1] = self.hostname
+                    hosts[i] = f"{ip_host[0]}\t{ip_host[1]}"
+                    replaced = True
+
+                elif ip_host[1] == self.hostname:
+                    ip_host[0] = ip
+                    hosts[i] = f"{ip_host[0]}\t{ip_host[1]}"
+                    replaced = True
+        if not replaced:
+            hosts.append(f"{ip}\t{self.hostname}")
+
+        config = ""
+        for line in hosts:
+            config = config + line + '\n'
+
+        sudo_writefile('/etc/hosts', config)
+
+    @windows_not_supported
+    def set_static_ip(self, ip, iface="eth0", mask="24"):
         """
         Sets the static ip on the sd card for the specified interface
         Also writes to master hosts file for easy access
@@ -462,77 +571,141 @@ class Burner(object):
         :return:
         :rtype:
         """
+        # TODO:
+        # router_ip statically set to default ip configured with cms bridge create. Rewrite to consider the IP of the master on iface
 
-        # Adds the ip and hostname to /etc/hosts if it isn't already there.
-        def add_to_hosts(ip):
-            # with open('/etc/hosts', 'r') as host_file:
-            #     hosts = host_file.readlines()
-            hosts = sudo_readfile('/etc/hosts')
-
-            replaced = False
-            for i in range(len(hosts)):
-                ip_host = hosts[i].split()
-
-                if len(ip_host) > 1:
-                    if ip_host[0] == ip:
-                        ip_host[1] = self.hostname
-                        hosts[i] = f"{ip_host[0]}\t{ip_host[1]}\n"
-                        replaced = True
-
-                    elif ip_host[1] == self.hostname:
-                        ip_host[0] = ip
-                        hosts[i] = f"{ip_host[0]}\t{ip_host[1]}\n"
-                        replaced = True
-            if not replaced:
-                hosts.append(f"{ip}\t{self.hostname}\n")
-
-            # with open('/etc/hosts', 'w') as host_file:
-            #     host_file.writelines(hosts)
-            config = ""
-            for line in hosts:
-                config = config + line + '\n'
-
-            sudo_writefile('/etc/hosts', config)
-
-        # Add static IP and hostname to master's hosts file and configure worker with static IP
-        if not self.dryrun:
-            add_to_hosts(ip)
-
-            # Configure static LAN IP
-            if iface == "eth0":
-                interfaces_conf = textwrap.dedent(f"""
-                auto {iface}
-                iface {iface} inet static
-                    address {ip}/{mask}
-                """)
-                # with open(f'{mountpoint}/etc/network/interfaces',
-                #           'a') as config:
-                #     config.write(interfaces_conf)
-                sudo_writefile(f'{mountpoint}/etc/network/interfaces',
-                               interfaces_conf, append=True)
-
-            # Configure static wifi IP
-            elif iface == "wlan0":
-                dnss = \
-                    self.system("cat /etc/resolv.conf | grep nameserver").split()[
-                        1]  # index 0 is "nameserver" so ignore
-                routerss = self.system(
-                    "ip route | grep default | awk '{print $3}'")  # omit the \n at the end
-                dhcp_conf = textwrap.dedent(f"""
-                        interface wlan0
-                        static ip_address={ip}
-                        static routers={routerss}
-                        static domain_name_servers={dnss}
-                        """)
-                # with open(f'{mountpoint}/etc/dhcpcd.conf', 'a') as config:
-                #     config.write(dhcp_conf)
-                sudo_writefile(f'{mountpoint}/etc/dhcpcd.conf', dhcp_conf,
-                               append=True)
+        if os_is_pi():
+            card = SDCard()
+        elif os_is_linux():
+            card = SDCard(card_os="linux")
         else:
-            print('interface eth0\n')
-            print(f'static ip_address={ip}/{mask}')
+            raise NotImplementedError
 
-    def set_key(self, name, mountpoint):
+        mountpoint = card.root_volume
+        if self.hostname is not None:
+            self.add_to_hosts(ip)
+        router_ip = '10.1.1.1'
+
+        iface = f'interface {iface}'
+        static_ip = f'static ip_address={ip}/{mask}'
+        static_routers = f'static routers={router_ip}'
+
+        curr_config = sudo_readfile(f'{mountpoint}/etc/dhcpcd.conf')
+        if iface in curr_config:
+            Console.warning("Found previous settings. Overwriting")
+            # If setting already present, replace it and the static ip line
+            index = curr_config.index(iface)
+            try:
+                if 'static ip_address' not in curr_config[index + 1]:
+                    Console.warning(
+                        "Missing static ip_address assignment. Overwriting line")
+                curr_config[index + 1] = static_ip
+
+            except IndexError:
+                Console.error(f'{mountpoint}/etc/dhcpcd.conf ends abruptly. Aborting')
+                sys.exit(1)
+
+        else:
+            curr_config.append(iface)
+            curr_config.append(static_ip)
+            curr_config.append(static_routers)
+            curr_config.append('\n')
+            # curr_config.append('nolink\n')
+
+        sudo_writefile(f'{mountpoint}/etc/dhcpcd.conf', '\n'.join(curr_config))
+
+    # TODO:
+    # Deprecated function as dhcpcd.conf is the recommended file for configuring static network configs. Should we keep this?
+    #
+    # def set_static_ip2(self, ip, mountpoint, iface="eth0", mask="16"):
+    #     """
+    #     Sets the static ip on the sd card for the specified interface
+    #     Also writes to master hosts file for easy access
+
+    #     :param ip: ips address
+    #     :type ip: str
+    #     :param mountpoint: the mountpunt of the device on which the ip
+    #                        is found
+    #     :type mountpoint: str
+    #     :param iface: the network Interface
+    #     :type iface: str
+    #     :param mask: the subnet Mask
+    #     :type mask: str
+    #     :return:
+    #     :rtype:
+    #     """
+
+    #     # Adds the ip and hostname to /etc/hosts if it isn't already there.
+    #     def add_to_hosts(ip):
+    #         # with open('/etc/hosts', 'r') as host_file:
+    #         #     hosts = host_file.readlines()
+    #         hosts = sudo_readfile('/etc/hosts')
+
+    #         replaced = False
+    #         for i in range(len(hosts)):
+    #             ip_host = hosts[i].split()
+
+    #             if len(ip_host) > 1:
+    #                 if ip_host[0] == ip:
+    #                     ip_host[1] = self.hostname
+    #                     hosts[i] = f"{ip_host[0]}\t{ip_host[1]}\n"
+    #                     replaced = True
+
+    #                 elif ip_host[1] == self.hostname:
+    #                     ip_host[0] = ip
+    #                     hosts[i] = f"{ip_host[0]}\t{ip_host[1]}\n"
+    #                     replaced = True
+    #         if not replaced:
+    #             hosts.append(f"{ip}\t{self.hostname}\n")
+
+    #         # with open('/etc/hosts', 'w') as host_file:
+    #         #     host_file.writelines(hosts)
+    #         config = ""
+    #         for line in hosts:
+    #             config = config + line + '\n'
+
+    #         sudo_writefile('/etc/hosts', config)
+
+    #     # Add static IP and hostname to master's hosts file and configure worker with static IP
+    #     if not self.dryrun:
+    #         add_to_hosts(ip)
+
+    #         # Configure static LAN IP
+    #         if iface == "eth0":
+    #             interfaces_conf = textwrap.dedent(f"""
+    #             auto {iface}
+    #             iface {iface} inet static
+    #                 address {ip}/{mask}
+    #             """)
+    #             # with open(f'{mountpoint}/etc/network/interfaces',
+    #             #           'a') as config:
+    #             #     config.write(interfaces_conf)
+    #             sudo_writefile(f'{mountpoint}/etc/network/interfaces',
+    #                            interfaces_conf, append=True)
+
+    #         # Configure static wifi IP
+    #         elif iface == "wlan0":
+    #             dnss = \
+    #                 self.system("cat /etc/resolv.conf | grep nameserver").split()[
+    #                     1]  # index 0 is "nameserver" so ignore
+    #             routerss = self.system(
+    #                 "ip route | grep default | awk '{print $3}'")  # omit the \n at the end
+    #             dhcp_conf = textwrap.dedent(f"""
+    #                     interface wlan0
+    #                     static ip_address={ip}
+    #                     static routers={routerss}
+    #                     static domain_name_servers={dnss}
+    #                     """)
+    #             # with open(f'{mountpoint}/etc/dhcpcd.conf', 'a') as config:
+    #             #     config.write(dhcp_conf)
+    #             sudo_writefile(f'{mountpoint}/etc/dhcpcd.conf', dhcp_conf,
+    #                            append=True)
+    #     else:
+    #         print('interface eth0\n')
+    #         print(f'static ip_address={ip}/{mask}')
+
+    @windows_not_supported
+    def set_key(self, name):
         """
         Copies the public key into the .ssh/authorized_keys file on the sd card
 
@@ -544,27 +717,36 @@ class Burner(object):
         """
         # copy file on burner computer ~/.ssh/id_rsa.pub into
         #   mountpoint/home/pi/.ssh/authorized_keys
-        self.system(f'mkdir -p {mountpoint}/home/pi/.ssh/')
-        self.system(
-            f'cp {name} {mountpoint}/home/pi/.ssh/authorized_keys')
+        if os_is_pi():
+            card = SDCard()
+        elif os_is_linux():
+            card = SDCard(card_os="linux")
+        else:
+            raise NotImplementedError
 
-    def mount(self, device, mountpoint="/mount/pi"):
+        mountpoint = card.root_volume
+        self.system(f'mkdir -p {mountpoint}/home/pi/.ssh/')
+        self.system(f'cp {name} {mountpoint}/home/pi/.ssh/authorized_keys')
+
+    @windows_not_supported
+    def mount(self, device=None, card_os="raspberry", host=None):
         """
         Mounts the current SD card
-
-        :param device: device to mount, e.g. /dev/sda
-        :type device: str
-        :param mountpoint: the mountpunt of the device on which the pi user
-                           is found
-        :type mountpoint: str
         """
-        if os_is_linux():
-            card = SDCard(os="raspberry", host="ubuntu")
-            dmesg = USB.get_from_dmesg()
+        host = host or get_platform()
+        card = SDCard(card_os=card_os,host=host)
+        dmesg = USB.get_from_dmesg()
+
+        #TODO Need a better way to itentify which sd card to use for mounting
+        # instead of iterating over all of them
+
+        if not self.dryrun:
+            self.system('sudo sync')  # flush any pending/in-process writes
 
             for usbcard in dmesg:
 
-                dev = usbcard['dev']
+                dev = device or usbcard['dev']
+                print(dev)
                 sd1 = f"{dev}1"
                 sd2 = f"{dev}2"
                 try:
@@ -582,85 +764,66 @@ class Burner(object):
                 except Exception as e:
                     print(e)
 
-        if os_is_pi():
-
-            # mount p2 (/) and then p1 (/boot)
-
-            if not self.dryrun:
+        #Keeping in case this was needed. Worked without it in testing.
+        #elif os_is_pi():
+        #    if not self.dryrun:
                 # wait for the OS to detect the filesystems
-                # in burner.info(), formatted will be true if the card has FAT32
+                # in burner.info(), formatted will be true if the card has
+                #        FAT32
                 #   filesystems on it
-                counter = 0
-                max_tries = 5
-                b = Burner()
-                while counter < max_tries:
-                    time.sleep(1)
-                    formatted = b.info(print_stdout=False)[device]['formatted']
-                    if formatted:
-                        break
-                    counter += 1
-                    if counter == max_tries:
-                        print("Timed out waiting for OS to detect filesystem"
-                              " on the burned card")
-                        sys.exit(1)
+        #        counter = 0
+        #        max_tries = 5
+        #        b = Burner()
+        #        while counter < max_tries:
+        #            time.sleep(1)
+        #            formatted = b.info(print_stdout=False)[device]['formatted']
+        #            if formatted:
+        #                break
+        #            counter += 1
+        #            if counter == max_tries:
+        #                print("Timed out waiting for OS to detect filesystem"
+        #                      " on the burned card")
+        #                sys.exit(1)
 
-            self.system(f'sudo mkdir -p {mountpoint}')
-            self.system(f'sudo mount {device}2 {mountpoint}')
-            self.system(f'sudo mount {device}1 {mountpoint}/boot')
-
-    def unmount(self, device=None):
+    @windows_not_supported
+    def unmount(self, device=None, card_os="raspberry", host=None):
         """
         Unmounts the current SD card
 
         :param device: device to unmount, e.g. /dev/sda
         :type device: str
         """
+
+        host = host or get_platform()
+        card = SDCard(card_os=card_os, host=host)
+
         if not self.dryrun:
             self.system('sudo sync')  # flush any pending/in-process writes
 
-        if device is None:
+            os.system(f"sudo umount {card.boot_volume}")
+            time.sleep(3)
+            os.system(f"sudo umount {card.root_volume}")
 
-            if os_is_linux():
-                user = os.environ["USER"]
-                os.system(f"sudo umount /media/{user}/boot")
-                os.system(f"sudo rmdir /media/{user}/boot")
+            time.sleep(3)
 
-                os.system(f"sudo umount /media/{user}/rootfs")
-                os.system(f"sudo rmdir /media/{user}/rootfs")
+            rm = [f"sudo rmdir {card.boot_volume}",
+                  f"sudo rmdir {card.root_volume}"]
 
-            else:
-                Console.error("Please specify the mount path")
-        else:
+            for command in rm:
+                print(rm)
+                os.system(command)
 
-            # unmount p1 (/boot) and then p2 (/)
-            self.system(f'sudo umount {device}1')
-            # noinspection PyBroadException
-            # try:
-            #     print(f"trying to unmount {device}1")
-            #     self.system(f'sudo umount {device}1')
-            # except:
-            #     pass
-
-            # Occasionally there are issues with unmounting. Pause for good effect.
-            self.system('sleep 1')
-            self.system(f'sudo umount {device}2')
-
-    def enable_ssh(self, mountpoint):
+    @windows_not_supported
+    def enable_ssh(self):
         """
         Enables ssh on next boot of sd card
-
-        :param mountpoint: mount point of the /boot file system wher the
-                           ssh is enabled. by adding a file ssh
-        :type mountpoint: str
         """
         if os_is_pi():
-            # touch mountpoint/boot/ssh
-            command = f'sudo touch {mountpoint}/boot/ssh'
+            card = SDCard(card_os="raspberry")
+            command = f'sudo touch {card.boot_volume}/ssh'
             self.system(command)
         elif os_is_linux():
-            card = SDCard(os="raspberry", host="ubuntu")
-
-            print("OOOO")
+            card = SDCard(card_os="raspberry")
             command = f"sudo touch {card.boot_volume}/ssh"
             self.system(command)
 
@@ -670,9 +833,11 @@ class Burner(object):
     # IMPROVE
 
     # TODO: docstring
-    def disable_password_ssh(self, mountpoint):
+    @windows_not_supported
+    def disable_password_ssh(self):
         # sshd_config = self.filename("/etc/ssh/sshd_config")
-        sshd_config = f'{mountpoint}/etc/ssh/sshd_config'
+        card = SDCard(card_os="raspberry")
+        sshd_config = f'{card.root_volume}/etc/ssh/sshd_config'
         new_sshd_config = ""
         updated_params = False
 
@@ -727,6 +892,7 @@ class Burner(object):
 
     # IMPROVE
     # ok osx
+    @windows_not_supported
     def activate_ssh(self, public_key, debug=False, interactive=False):
         """
         Sets the public key path and copies it to the SD card
@@ -826,10 +992,10 @@ class Burner(object):
                 f.write(new_rc_local)
         self.disable_password_ssh()
 
+    @windows_not_supported
     def configure_wifi(self,
                        ssid,
                        psk=None,
-                       mountpoint='/mount/pi',
                        interactive=False):
         """
         Sets the wifi. Only works for psk based wifi
@@ -838,8 +1004,6 @@ class Burner(object):
         :type ssid: str
         :param psk: the psk
         :type psk: str
-        :param mountpoint: the mont pount
-        :type mountpoint: str
         :param interactive: true if you like to run it interactively
         :type interactive: bool
         """
@@ -866,8 +1030,8 @@ class Burner(object):
                     }}""".format(network=ssid))
 
         # Per fix provided by Gregor, we use this path to get around rfkill block on boot
-        path = f"{mountpoint}/boot/wpa_supplicant.conf"
-        # path = f"{mp}/etc/wpa_supplicant/wpa_supplicant.conf"
+        card = SDCard(card_os=card_os, host=host)
+        path = f"{card.boot_volume}/wpa_supplicant.conf"
         if self.dryrun:
             print("DRY RUN - skipping:")
             print("Writing wifi ssid:{} psk:{} to {}".format(ssid,
@@ -879,13 +1043,11 @@ class Burner(object):
         sudo_writefile(path, wifi)
 
     # TODO
-    def format_device(self, device='dev/sda', hostname=None):
+    @windows_not_supported
+    def format_device(self, device='dev/sdX', hostname=None, title="UNTITLED"):
         """
         Formats device with one FAT32 partition
 
-        WARNING: This is a potential risky way of automating
-                 the process using fdisk. ONly use on SDCards that you can lose.
-
         WARNING: make sure you have the right device, this comamnd could
                  potentially erase your OS
 
@@ -894,43 +1056,46 @@ class Burner(object):
         :param hostname: the hostname
         :type hostname: str
         """
-        if not os_is_pi():
-            raise NotImplementedError("Only implemented to be run on a PI")
 
-        Console.info("Formatting device...")
-        self.unmount(device)
-        StopWatch.start(f"format {device} {hostname}")
+        if os_is_linux() or os_is_pi():
 
-        pipeline = textwrap.dedent("""d
+            banner("format {device}")
 
-                                    d
-                                    
-                                    d
+            script = f"""
+                sudo eject -t {device}
+                sudo parted {device} --script -- mklabel msdos
+                sudo parted {device} --script -- mkpart primary fat32 1MiB 100%
+                sudo mkfs.vfat -n {title} -F32 {device}1
+                sudo parted {device} --script print""".strip().splitlines()
+            for line in script:
+                print(line)
+                os.system(line)
 
-                                    d
+            Console.ok("Formatted SD Card")
 
-                                    n
-                                    p
-                                    1
+        else:
+            raise NotImplementedError("Not implemented for this OS")
 
+    @windows_not_supported
+    def load_device(self, device='dev/sdX'):
+        """
+        Loads the USB device via trayload
 
-                                    t
-                                    b""")
+        :param device: The defice on which we format
+        :type device: str
+        :param hostname: the hostname
+        :type hostname: str
+        """
+        if os_is_pi():
+            Console.error("Not yet implemented")
 
-        command = f'(echo "{pipeline}"; sleep 1; echo "w") | sudo fdisk {device}'
-        result = subprocess.getoutput(command)
+        elif os_is_linux():
 
-        StopWatch.stop(f"format {device} {hostname}")
-        StopWatch.status(f"format {device} {hostname}", True)
+            banner(f"load {device}")
+            os.system(f"sudo eject -t {device}")
 
-        #
-        # TODO: we should have a test here
-        #
-        StopWatch.benchmark(sysinfo=True, csv=False)
-
-        Console.ok("Formating completed ...")
-
-        Console.info("Wait while the card is written ...")
+        else:
+            raise Console.error("Not implemented for this OS")
 
     # This is to prevent desktop access of th pie (directly plugging monitor, keyboard, mouse into pi, etc.)
     #
@@ -938,6 +1103,7 @@ class Burner(object):
     # Plugging pi directly into desktop, however, will still prompt for a user and password.
     # I can't figure out how to disable it
 
+    @windows_not_supported
     def disable_terminal_login(self, mountpoint, password):
         """
         disables and replaces the password with a random string so that by
@@ -1039,7 +1205,8 @@ class MultiBurner(object):
                  password=None,
                  ssid=None,
                  psk=None,
-                 fromatting=True):
+                 fromatting=True,
+                 tag='latest-lite'):
         """
         TODO: provide documentation
 
@@ -1140,7 +1307,7 @@ class MultiBurner(object):
             ip = None if not ips else ips[i]
 
             self.burn(image, device, blocksize, progress, hostname,
-                      ip, key, password, ssid, psk, fromatting)
+                      ip, key, password, ssid, psk, fromatting, tag)
 
             count += 1
             Console.info(f'Burned card {count}')
@@ -1172,7 +1339,8 @@ class MultiBurner(object):
              password=None,
              ssid=None,
              psk=None,
-             fromatting=True):
+             fromatting=True,
+             tag='latest-lite'):
         """
         Burns the image on the specific device
 
@@ -1203,8 +1371,10 @@ class MultiBurner(object):
         :return:
         :rtype:
         """
-
-        mp = '/mount/pi'
+        #TODO
+        card = SDCard()
+        boot_volume = card.boot_volume
+        root_volume = card.root_volume
         if key is None:
             key = '~/.ssh/id_rsa.pub'
 
@@ -1226,24 +1396,24 @@ class MultiBurner(object):
         if fromatting:
             burner.format_device(device=device, hostname=hostname)
 
-        burner.burn(image, device, blocksize=blocksize)
-        burner.mount(device, mp)
-        burner.set_hostname(hostname, mp)
-        burner.disable_terminal_login(mp, password)
+        burner.burn_sdcard(tag=tag, device=device, blocksize=blocksize)
+        burner.mount(device=device)
+        burner.set_hostname(hostname)
+        burner.disable_terminal_login(root_volume, password)
         if ssid:
-            Console.warning(
-                "Warning: In the future, try to interface with the workers via ethernet/switch rather than WiFi")
-            burner.configure_wifi(ssid, psk, mp)
-        burner.enable_ssh(mp)
-        burner.disable_password_ssh(mp)
-        burner.set_key(key, mp)
+            Console.warning("In the future, try to interface with the workers via "
+                            "ethernet/switch rather than WiFi")
+            burner.configure_wifi(ssid, psk)
+        burner.enable_ssh()
+        burner.disable_password_ssh()
+        burner.set_key(key)
         if ip:
             interface = "wlan0" if ssid is not None else "eth0"
-            burner.set_static_ip(ip, mp, iface=interface)
+            burner.set_static_ip(ip, iface=interface)
 
         burner.unmount(device)
         # for some reason, need to do unmount twice for it to work properly
-        burner.unmount(device)
+        # burner.unmount(device)
         time.sleep(2)
         StopWatch.stop(f"create {device} {hostname}")
         StopWatch.status(f"create {device} {hostname}", True)
