@@ -1,18 +1,14 @@
 import os
 from getpass import getpass
-from pathlib import Path
 
-import oyaml as yaml
 from cloudmesh.burn.burner import Burner
 from cloudmesh.burn.burner import MultiBurner
 from cloudmesh.burn.burner import gen_strong_pass
 from cloudmesh.burn.image import Image
 from cloudmesh.burn.network import Network
-from cloudmesh.burn.util import readfile
-from cloudmesh.burn.util import writefile
+from cloudmesh.burn.util import os_is_pi
 from cloudmesh.common.StopWatch import StopWatch
 from cloudmesh.common.Tabulate import Printer
-from cloudmesh.common.debug import VERBOSE
 from cloudmesh.common.parameter import Parameter
 from cloudmesh.common.util import Console
 from cloudmesh.shell.command import PluginCommand
@@ -28,17 +24,21 @@ class BurnCommand(PluginCommand):
         ::
 
             Usage:
+              burn firmware check
+              burn install
+              burn load --device=DEVICE
+              burn format --device=DEVICE
+              burn mount [--device=DEVICE] [--os=OS]
+              burn unmount [--device=DEVICE] [--os=OS]
               burn network list [--ip=IP] [--used]
               burn network
               burn info [--device=DEVICE]
-              burn detect
-              burn image versions [--refresh]
+              burn image versions [--refresh] [--yaml]
               burn image ls
               burn image delete [--image=IMAGE]
-              burn image get [--url=URL]
+              burn image get [--url=URL] [TAG...]
               burn backup [--device=DEVICE] [--to=DESTINATION]
               burn copy [--device=DEVICE] [--from=DESTINATION]
-              burn shrink install
               burn shrink [--image=IMAGE]
               burn create [--image=IMAGE]
                           [--device=DEVICE]
@@ -51,15 +51,14 @@ class BurnCommand(PluginCommand):
                           [--ssid=SSID]
                           [--wifipassword=PSK]
                           [--format]
-              burn sdcard [--image=IMAGE] [--device=DEVICE] [--dryrun]
-              burn mount [--device=DEVICE] [--mount=MOUNTPOINT]
+                          [--tag=TAG]
+              burn sdcard [TAG...] [--device=DEVICE] [--dryrun]
               burn set [--hostname=HOSTNAME]
                        [--ip=IP]
                        [--key=KEY]
-                       [--mount=MOUNTPOINT]
-              burn enable ssh [--mount=MOUNTPOINT]
-              burn unmount [--device=DEVICE]
-              burn wifi SSID [--passwd=PASSWD] [-ni]
+              burn enable ssh
+              burn wifi --ssid=SSID [--passwd=PASSWD] [-ni]
+              burn check [--device=DEVICE]
 
             Options:
               -h --help              Show this screen.
@@ -72,6 +71,9 @@ class BurnCommand(PluginCommand):
               --key=KEY              The name of the SSH key file
               --blocksize=BLOCKSIZE  The blocksise to burn [default: 4M]
 
+            Arguments:
+                TAG                  Keyword tags to identify an image
+                                     [default: latest]
             Files:
               This is not fully thought through and needs to be documented
               ~/.cloudmesh/images
@@ -122,6 +124,115 @@ class BurnCommand(PluginCommand):
                      | wlan0   | 192.168.1.12   | 192.168.1.255  |
                      +---------+----------------+----------------+
 
+                cms burn firmware check
+
+                    checks if the firmware on the Pi is up to date
+
+                cms burn install
+
+                    installs a program to shring img files. THis is useful, after
+                    you created a backup to make the backup smaller and allow
+                    faster burning in case of recovery
+
+                cms burn load --device=DEVICE
+
+                    loads the sdcard into the USB drive. Thi sis similar to
+                    loading a cdrom drive. It s the oposite to eject
+
+                cms burn format --device=DEVICE
+
+                    formats the SDCard in the specified device. Be careful it is
+                    the correct device.  cms burn info will help you to identifying it
+
+                cms burn mount [--device=DEVICE] [--os=OS]
+
+                    mounts the file systems available on the SDCard
+
+                cms burn unmount [--device=DEVICE] [--os=OS]
+
+                    unmounts the mounted file systems from the SDCard
+
+                cms burn info [--device=DEVICE]
+
+                    provides useful information about the SDCard
+
+                cms burn image versions [--refresh] [--yaml]
+
+                    The images that you like to burn onto your SDCard can be cached locally with the image command.
+                    The available images for the PI can be found when using the --refresh option. If you do not
+                    specify it it reads a copy of the image list from our cache
+
+                cms burn image ls
+
+                    Lists all downloaded images in our cache. You can download
+                    them with the cms burn image get command
+
+                cms burn image delete [--image=IMAGE]
+
+                    deletes the specified image. The name can be found with the image ls command
+
+                cms burn image get [--url=URL] [TAG...]
+
+                    downloads a specific image or the latest image. The tag are a number of words
+                    separated by a space that must occur in the tag that you find in the versions command
+
+                cms burn backup [--device=DEVICE] [--to=DESTINATION]
+
+                    backs up a SDCard to the given location
+
+                cms burn copy [--device=DEVICE] [--from=DESTINATION]
+
+                    copies the file form the destination on the SDCard
+                    this is the same as the SDCard command. we will in future remove one
+
+                cms burn shrink [--image=IMAGE]
+
+                    shrinks the size of a backoup or image file that is on
+                    your local file system. It can only be used for .img files
+
+                cms burn create [--image=IMAGE]
+                                [--device=DEVICE]
+                                [--hostname=HOSTNAME]
+                                [--ip=IP]
+                                [--sshkey=KEY]
+                                [--blocksize=BLOCKSIZE]
+                                [--dryrun]
+                                [--passwd=PASSWD]
+                                [--ssid=SSID]
+                                [--wifipassword=PSK]
+                                [--format]
+
+                    This is a comprehensif cuntion that not only can format the SDCard, but also
+                    initializes it with specific falues
+
+
+                cms burn sdcard [TAG...] [--device=DEVICE] [--dryrun]
+
+                    this burns the sd card, see also copy and create
+
+                cms burn set [--hostname=HOSTNAME]
+                             [--ip=IP]
+                             [--key=KEY]
+                             [--mount=MOUNTPOINT]
+
+                    this sets specific values on the sdcard after it has ben created
+                    with the creat, copy or sdcard command
+
+                    a --ssh is missing from this command
+
+                cms burn enable ssh [--mount=MOUNTPOINT]
+
+                    this enables the ssh server once it is booted
+
+                cms burn wifi --ssid=SSID [--passwd=PASSWD] [-ni]
+
+                    this sets the wifi ssid and password afterthe card is created,
+                    copies, or sdcard is used
+
+                cms burn check [--device=DEVICE]
+
+                    this command lists the parameters that were set with the set or create command
+
             Examples: ( \\ is not shown)
 
                > cms burn create --image=2019-09-26-raspbian-buster-lite
@@ -154,19 +265,21 @@ class BurnCommand(PluginCommand):
                        "passwd",
                        "wifipassword",
                        "version",
-                       "to")
-        arguments.mountpoint = arguments["--mount"]
+                       "to",
+                       "os")
+        # arguments.MOUNTPOINT = arguments["--mount"]
         arguments.FORMAT = arguments["--format"]
         arguments.FROM = arguments["--from"]
         arguments.IMAGE = arguments["--image"]
 
-        VERBOSE(arguments)
+        # VERBOSE(arguments)
 
         def execute(label, function):
             StopWatch.start(label)
-            function
+            result = function
             StopWatch.stop(label)
             StopWatch.status(label, True)
+            return result
 
         dryrun = arguments['--dryrun']
 
@@ -175,56 +288,51 @@ class BurnCommand(PluginCommand):
         StopWatch.stop("info")
         StopWatch.status("info", True)
 
-        if arguments.versions and arguments['image']:
+        if arguments.firmware and arguments.check:
+
+            execute("firmware check", burner.firmware(action="check"))
+            return ""
+
+        elif arguments.firmware and arguments.update:
+
+            execute("firmware update", burner.firmware(action="update"))
+            return ""
+
+        if arguments.check:
+
+            execute("check", burner.check(device=arguments.device))
+            return ""
+
+        elif arguments.versions and arguments['image']:
 
             StopWatch.start("image versions")
-            image = Image()
-            data = {
-                "lite": [],
-                "full": []
-            }
-            cache = Path(
-                os.path.expanduser("~/.cloudmesh/cmburn/distributions.yaml"))
-            if arguments["--refresh"] or not cache.exists():
-                os.system("mkdir -p ~/.cloudmesh/cmburn")
-                print("finding lite repos ...", end="")
-                repos = [f"{image.raspberry_lite_images}"]
-                for repo in repos:
-                    versions, downloads = Image().versions(repo)
-                    print("These images are available at")
-                    for version, download in zip(versions, downloads):
-                        entry = {
-                            "version": version,
-                            "url": download,
-                            "date": version.split("-", 1)[1]
-                        }
-                        data["lite"].append(entry)
 
-                print("finding lite repos ...", end="")
-                repos = [f"{image.raspberry_full_images}"]
-                for repo in repos:
-                    versions, downloads = Image().versions(repo)
-                    print("These images are available at")
-                    for version, download in zip(versions, downloads):
-                        entry = {
-                            "version": version,
-                            "url": download,
-                            "date": version.split("-", 1)[1]
-                        }
-                        data["full"].append(entry)
-                writefile(cache, yaml.dump(data))
-            else:
-                # data = yaml.load(readfile(cache), Loader=yaml.SafeLoader)
-                # for entry in data:
-                #   version = list(entry.keys())[0]
-                #    download = entry[version]
-                #    print(f"{version}: {download}")
-                data = readfile(cache)
-                print(data)
+            result = Image.create_version_cache(refresh=arguments["--refresh"])
+
+            output = "table"
+            if arguments["--yaml"]:
+                output = "yaml"
+
+            print(Printer.write(
+                result,
+                order=["tag", 'date', "type", 'version', "url"],
+                header=["Tag", 'Date', "Type", 'Version', "Url"],
+                output=output
+            )
+            )
+
             StopWatch.stop("image versions")
             StopWatch.status("image versions", True)
             return ""
 
+        elif arguments.load:
+            execute("load", burner.load_device(device=arguments.device))
+            return ""
+
+        elif arguments["format"]:  # as format is a python word, we need to use an index
+            execute("format", burner.format_device(device=arguments.device))
+            return ""
+
         elif arguments.network and arguments["list"]:
 
             ip = arguments.ip or Network.address()[0]['local']
@@ -253,12 +361,15 @@ class BurnCommand(PluginCommand):
                 details,
                 order=['label', "local", "broadcast"],
                 header=["Label", "Local", "Broadcast"]
-            )
+                )
             )
             return ""
 
         elif arguments.wifi:
 
+            Console.error("This command is not yet implemented")
+            return ""
+
             password = arguments.passwd
             ssid = arguments.ssid
 
@@ -271,19 +382,14 @@ class BurnCommand(PluginCommand):
             StopWatch.status("wifi", True)
             return ""
 
-        elif arguments.detect:
-
-            execute("detect", burner.detect())
-            return ""
-
         elif arguments.info:
 
             execute("info", burner.info())
             return ""
 
-        elif arguments.shrink and arguments.install:
+        elif arguments.install:
 
-            execute("shrink install", burner.shrink_install())
+            execute("install", burner.install())
             return ""
 
         elif arguments.shrink:
@@ -292,69 +398,45 @@ class BurnCommand(PluginCommand):
             return ""
 
         elif arguments.backup:
-
-            execute("backup",
-                    burner.backup(device=arguments.device, to_file=arguments.to))
+            execute("backup", burner.backup(device=arguments.device, to_file=arguments.to))
             return ""
 
-        elif arguments.copy:
-
-            execute("copy",
-                    burner.copy(device=arguments.device, from_file=arguments.FROM))
+        elif arguments["copy"]:  # as copy is a reserved word we need to use the index
+            execute("copy", burner.copy(device=arguments.device, from_file=arguments.FROM))
             return ""
 
         elif arguments.sdcard:
-            # check_root(dryrun=dryrun)
+            arguments.TAG = arguments.TAG or ["latest"]
 
-            image = arguments.IMAGE
-            device = arguments.device
-            execute("sdcard", burner.burn_sdcard(image, device))
+            execute("sdcard", burner.burn_sdcard(tag=arguments.TAG, device=arguments.device))
             return ""
 
         elif arguments.mount:
-            # check_root(dryrun=dryrun)
+            execute("mount", burner.mount(device=arguments.device,
+                                          card_os=arguments.os))
+            return ""
 
-            device = arguments.device
-            mp = arguments.mountpoint
-            execute("mount", burner.mount(device, mp))
+        elif arguments.unmount:
+            execute("unmount", burner.unmount(device=arguments.device,
+                                              card_os=arguments.os))
             return ""
 
         elif arguments.set:
 
-            if arguments.host:
-                # check_root(dryrun=dryrun)
-
-                hostname = arguments.hostname
-                mp = arguments.mountpoint
-                execute("set hostname", burner.set_hostname(hostname, mp))
+            if arguments.hostname:
+                execute("set hostname", burner.set_hostname(arguments.hostname))
 
             if arguments.ip:
-                # check_root(dryrun=dryrun)
-
-                ip = arguments.ip
-                mp = arguments.mountpoint
-                execute("set ip", burner.set_static_ip(ip, mp))
+                execute("set ip", burner.set_static_ip(arguments.ip))
 
             if arguments.key:
-                # check_root(dryrun=dryrun)
+                execute("set key", burner.set_key(arguments.key))
 
-                key = arguments.key
-                mp = arguments.mountpoint
-                execute("set key", burner.set_key(key, mp))
-                return ""
-
-        elif arguments.enable and arguments.ssh:
-            # check_root(dryrun=dryrun)
-
-            mp = arguments.mountpoint
-            execute("enable ssh", burner.enable_ssh(mp))
             return ""
 
-        elif arguments.unmount:
-            # check_root(dryrun=dryrun)
+        elif arguments.enable and arguments.ssh:
 
-            device = arguments.device
-            execute("unmount", burner.unmount(device))
+            execute("enable ssh", burner.enable_ssh())
             return ""
 
         # elif arguments.versions and arguments.image:
@@ -365,11 +447,22 @@ class BurnCommand(PluginCommand):
             return ""
 
         elif arguments.delete and arguments.IMAGE:
-            execute("image rm", Image(arguments.IMAGE).rm())
+            execute("image rm", Image().rm(arguments.IMAGE))
             return ""
 
-        elif arguments.get and arguments['image']:
-            execute("image fetch", Image(arguments.url).fetch())
+        elif arguments["get"] and arguments['image'] and arguments["--url"]:
+            image = Image()
+            execute("image fetch", image.fetch(url=arguments.url))
+            return ""
+
+        elif arguments["get"] and arguments['image'] and arguments["TAG"]:
+            image = Image()
+            execute("image fetch", image.fetch(tag=arguments["TAG"]))
+            return ""
+
+        elif arguments["get"] and arguments['image']:
+            image = Image()
+            execute("image fetch", image.fetch(tag="latest"))
             return ""
 
         elif arguments.create:
@@ -410,29 +503,33 @@ class BurnCommand(PluginCommand):
 
             ips = None if not arguments.ip else Parameter.expand(arguments.ip)
             key = arguments.sshkey
-            mp = '/mount/pi'
-            blocksize = arguments.blocksize
+            tag = arguments['--tag']
 
-            StopWatch.start("total")
+            if os_is_pi():
+                blocksize = arguments.blocksize
+                StopWatch.start("total")
 
-            multi = MultiBurner()
-            multi.burn_all(
-                image=image,
-                device=devices,
-                blocksize=blocksize,
-                progress=True,
-                hostnames=hostnames,
-                # not difference between names and name, maybe we shoudl allign
-                ips=ips,
-                key=key,
-                password=passwd,
-                ssid=ssid,
-                psk=psk)
+                multi = MultiBurner()
+                multi.burn_all(
+                    image=image,
+                    device=devices,
+                    blocksize=blocksize,
+                    progress=True,
+                    hostnames=hostnames,
+                    # not difference between names and name, maybe we shoudl allign
+                    ips=ips,
+                    key=key,
+                    password=passwd,
+                    ssid=ssid,
+                    psk=psk,
+                    tag=tag)
 
-            StopWatch.stop("total")
-            StopWatch.status("total", True)
+                StopWatch.stop("total")
+                StopWatch.status("total", True)
 
-            StopWatch.benchmark(sysinfo=False, csv=False)
+                StopWatch.benchmark(sysinfo=False, csv=False)
+            else:
+                Console.error("This command is only supported ona Pi")
             return ""
 
         Console.error("see manual page: cms help burn")
