@@ -148,6 +148,7 @@ class USB(object):
 
         lsusb = USB.get_from_lsusb()
         busses = usb.busses()
+
         details = []
         for bus in busses:
             devices = bus.devices
@@ -179,6 +180,7 @@ class USB(object):
         :rtype: list of dicts
         """
         lsusb = subprocess.getoutput("lsusb").splitlines()
+
         all = {}
         for line in lsusb:
             line = line.replace("Bus ", "")
@@ -187,19 +189,23 @@ class USB(object):
             line = line.replace(":", "", 1)
             line = line.replace(":", " ", 1)
             content = line.split(" ")
-            bus = int(content[0])
-            addr = int(content[1])
 
-            data = {
-                'bus': bus,
-                'addr': addr,
-                'ivendor': int(content[2], 16),
-                'iproduct': int(content[3], 16),
-                'vendor': content[2],
-                'product': content[3],
-                'comment': ' '.join(content[4:])
-            }
-            all[f"{bus}-{addr}"] = data
+            try:
+                bus = int(content[0])
+                addr = int(content[1])
+
+                data = {
+                    'bus': bus,
+                    'addr': addr,
+                    'ivendor': int(content[2], 16),
+                    'iproduct': int(content[3], 16),
+                    'vendor': content[2],
+                    'product': content[3],
+                    'comment': ' '.join(content[4:])
+                }
+                all[f"{bus}-{addr}"] = data
+            except Exception as e:
+                pass
 
         return all
 
@@ -224,10 +230,8 @@ class USB(object):
                 sdci, key, what = line.split(" ", 2)
                 comment = line.split("Direct-Access")[1].strip()
                 what = " ".join(comment.split("  ")[:2])
-                try:
+                if key not in details:
                     details[key] = {}
-                except Exception as e:
-                    pass
                 details[key]["key"] = key
                 details[key]["direct-access"] = True
                 details[key]["info"] = what
@@ -241,6 +245,8 @@ class USB(object):
 
             elif line.startswith("sd") and "] " in line:
                 prefix, key, device, comment = line.split(" ", 3)
+                if key not in details:
+                    details[key] = {}
                 details[key]["key"] = key
                 if "Attached SCSI removable disk" in comment:
                     details[key]["removable"] = True
@@ -251,6 +257,8 @@ class USB(object):
                                                                             "")
                 if "Write Protect is" in comment:
                     details[key]["writeable"] = "off" in comment
+                else:
+                    details[key]["writeable"] = True
                 name = details[key]["name"] = device.replace("[", "").replace(
                     "]", "")
                 dev = details[key]["dev"] = f"/dev/{name}"
